/*
 *  subproc.cpp
 *
 *  Run program as a subprocess.
 *
 *  Joris van Rantwijk 2024
 */

#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#include <system_error>

#include "subproc.hpp"


/** Run the specified program in a subprocess. */
int puzzlefw::run_subprocess(std::string path, std::vector<std::string> args)
{
    // Prepare argv array.
    std::vector<char*> argv_vec;
    argv_vec.push_back(path.data());
    for (std::string& arg : args) {
        argv_vec.push_back(arg.data());
    }
    argv_vec.push_back(nullptr);

    char** argv = argv_vec.data();

    int max_file_descriptor = sysconf(_SC_OPEN_MAX);

    // Block SIGCHLD.
    sigset_t block_sigmask, saved_sigmask;
    sigemptyset(&block_sigmask);
    sigaddset(&block_sigmask, SIGCHLD);
    pthread_sigmask(SIG_BLOCK, &block_sigmask, &saved_sigmask);

    pid_t pid = fork();
    if (pid == -1) {
        pthread_sigmask(SIG_SETMASK, &saved_sigmask, nullptr);
        throw std::system_error(
            std::error_code(errno, std::system_category()),
            std::string("Can not run subprocess"));
    }

    if (pid == 0) {
        // This code runs in the subprocess.

        // Restore signal mask.
        sigprocmask(SIG_SETMASK, &saved_sigmask, nullptr);

        // Close non-standard file descriptors.
        for (int i = 3; i < max_file_descriptor; i++) {
            close(i);
        }

        // Execute program.
        execv(argv[0], argv);
        _exit(127);
    }

    // Wait until the subprocess ends.
    int wstatus;
    while (1) {
        if (waitpid(pid, &wstatus, 0) < 0) {
            if (errno == EINTR) {
                continue;
            }
            pthread_sigmask(SIG_SETMASK, &saved_sigmask, nullptr);
            throw std::system_error(
                std::error_code(errno, std::system_category()),
                std::string("Can not wait for subprocess"));
        }
        break;
    }

    // Restore signal mask.
    pthread_sigmask(SIG_SETMASK, &saved_sigmask, nullptr);

    return wstatus;
}
